package uk.ac.rhul.cs.oo;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * The <code>Rhul</code> class features:
 * <ul>
 * <li>I/O operations,
 * <li>drawings,
 * <li>some string operations.
 * </ul>
 *
 * The <code>Rhul</code> class masks some more advanced Java constructs that are
 * not relevant in the context of a first programming course. These advanced
 * features will be uncovered in due time. The names in the <code>Rhul</code>
 * class have been chosen uniformly to ease programming.
 *
 * <H3>Input/Outputs (I/O)</H3>
 * <P>The inputs are always from the keyboard and the outputs are always shown
 * in the terminal.
 * <P> The methods
 * <CODE>print(<I>type</I>)</CODE> and
 * <CODE>println(<I>type</I>)</CODE> allow to print any data <I>type</I> 
 * (<CODE>println</CODE> adds a newline at the end).
 * <P> The input methods are of the form
 * <CODE>read<I>Type</I>()</CODE> for all the Java primitives data types.  In
 * case of an error, all these methods will update a flag that can be accessed
 * using the <CODE>isOk()</CODE> and <CODE>getStatus()</CODE>.
 *
 * <H3>Drawings</H3>
 *
 * <P>An example of graphical program is the following:
 * <PRE>
 * Deug.startDrawings(200,200);
 * for (i=0; i<200; i+=4) {
 *   Deug.setGray(i);
 *   Deug.drawLine(0,i,199,199-i);
 *   Deug.drawLine(i,0,199-i,199);
 * }
 * Deug.stopDrawings();
 * </PRE>
 * It produces:<BR>
 * <IMG SRC="Dessin.png">
 *
 * <H3>String operations</H3>
 * The provided methods are of the form <CODE><I>type</I>ToString()</CODE> and
 * <CODE>stringTo<I>type</I>()</CODE>
 * where (<I>type</I> can be:
 * <CODE>boolean</CODE>, <CODE>int</CODE>, <CODE>long</CODE>, <CODE>float</CODE>
 * ou <CODE>double</CODE>). These methods provide the expected conversions from
 * and to <CODE>String</CODE>.
 * <P>
 * The following comparison methods are also included: <CODE>equals()</CODE>,
 * <CODE>compare()</CODE>,
 * <CODE>subString()</CODE>,
 * <CODE>charAt()</CODE> et
 * <CODE>length()</CODE>.
 *
 * @author Jean-Baptiste.Yunes@liafa.jussieu.fr
 * @author Fabien.Tarissan@pps.jussieu.fr
 * @version 0.19, 30/10/08
 */
public final class Rhul {
    /**
     * The last I/O operation went well.
     */
    public static final int NO_ERROR=0;
    /**
     * Reading the input stream was impossible.
     */
    public static final int STREAM_ERROR=1;
    /**
     * The conversion of the character string that was read to the needed data
     * type was not possible.
     */
    public static final int FORMAT_ERROR=2;
    /**
     * The arguments is invalid (ex: <CODE>==null</CODE>).
     **/
    public static final int ARG_ERROR=3;
    /**
     * This class should not be instanciated.
     */
    private Rhul() {
    }
    /**
     * The lexeme tokeniser.
     */
    private static MyTokenizer theTokenizer =  new MyTokenizer(new InputStreamReader(System.in));
    /**
     * Internal error code.
     */
    private static int error;
    /**
     * This method return the error state of the last I/o operation.
     * <DL>
     * <DT><CODE>true</CODE><DD> if everything was fine;
     * <DT><CODE>false</CODE><DD> if an error occured. In that case, it is
     * useful to get the error code using <code>getStatus()</code>.
     * </DL>
     * Calling this method is equivalent to the expression
     * <CODE>getStatus()==NO_ERROR</CODE>.
     * @return if an I/O error occured or not.
     */
    public static boolean isOk() {
	return getStatus()==NO_ERROR;
    }
    /**
     * Returns the error code of the last I/O operation.
     * @return the error code of the last I/O operation.
     */
    public static int getStatus() {
	return error;
    }

    /**
     * Reads a character from the keyboard and returns its value.
     *
     * <P>The read bytes come from the standard input.
     *
     * @return the character that was entered. If the character is
     * <code>&lt;nul&gt;</code> it is advised to look if the I/O operation was
     * actually successful or not (<code>isOk()</code>).
     */
    public static char readChar() {
	int c;
	try {
	    c = theTokenizer.read();
	    if (c!=MyTokenizer.EOF) {
		error = NO_ERROR;
		return (char)c;
	    } else {
		error = STREAM_ERROR;
		return '\u0000';
	    }
	} catch(java.io.IOException ioe) {
	    error = STREAM_ERROR;
	    return '\u0000';
	}
    }

    /**
     * Terminates the program.
     */
    public static void exit() {
	System.exit(0);
    }

    /**
     * Read a character string until the next carriage return (end of line).
     *
     * <P>The characters are read one after the other using the 
     * <CODE>readChar()</CODE> method.
     * @return the next input line.
     */
    public static String readLine() {
	StringBuffer theString=new StringBuffer();
	char c;
	while ( ((c=readChar())!='\n') && isOk() ) {
	    theString.append(c);
	}
	if (!isOk()) return null;
	return theString.toString();
    }

    /**
     * Converts a character string into a <CODE>boolean</CODE>.
     * @param s the character string
     * @return the value after conversion (<CODE>boolean</CODE>) <DL>
     * <DT><CODE>true</CODE><DD>if <CODE>s</CODE> is equal to
     * <CODE>"true"</CODE>
     * <DT><CODE>false</CODE><DD>otherwise
     * </DL>
     **/
    public static boolean stringToBoolean(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return false;
	}
        try {
            return Boolean.valueOf(s).booleanValue();
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return false;
        }
    }
    /**
     * Converts a character string into an <CODE>int</CODE>.
     * <P>
     * In case of an error, the error state is updated.
     * @param s the character string
     * @return the value after conversion (<CODE>int</CODE>)
     * <B>Warning</B>: if the returned value is <CODE>0</CODE>, it is 
     * necessary to consult the error status of the operation.
     **/
    public static int stringToInt(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (int)0;
	}
        try {
            return Integer.parseInt(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (int)0;
        }
    }
    /**
     * Converts a character string into a <CODE>byte</CODE>.
     * <P>
     * In case of an error, the error state is updated.
     * @param s the character string
     * @return the value after conversion (<CODE>byte</CODE>)
     * <B>Warning</B>: if the returned value is <CODE>0</CODE>, it is 
     * necessary to consult the error status of the operation.
     **/
    public static byte stringToByte(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (byte)0;
	}
        try {
            return Byte.parseByte(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (byte)0;
        }
    }
    /**
     * Converts a character string into a <CODE>double</CODE>.
     * <P>
     * In case of an error, the error state is updated.
     * @param s the character string
     * @return the value after conversion (<CODE>double</CODE>)
     * <B>Warning</B>: if the returned value is <CODE>0.0</CODE>, it is 
     * necessary to consult the error status of the operation.
     **/
    public static double stringToDouble(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (double)0;
	}
        try {
            return Double.parseDouble(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (double)0;
        }
    }
    /**
     * Converts a character string into a <CODE>float</CODE>.
     * <P>
     * In case of an error, the error state is updated.
     * @param s the character string
     * @return the value after conversion (<CODE>float</CODE>)
     * <B>Warning</B>: if the returned value is <CODE>0.0</CODE>, it is 
     * necessary to consult the error status of the operation.
     **/
    public static float stringToFloat(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (float)0;
	}
        try {
            return Float.parseFloat(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (float)0;
        }
    }
    /**
     * Converts a character string into a <CODE>long</CODE>.
     * <P>
     * In case of an error, the error state is updated.
     * @param s the character string
     * @return the value after conversion (<CODE>long</CODE>)
     * <B>Warning</B>: if the returned value is <CODE>0</CODE>, it is 
     * necessary to consult the error status of the operation.
     **/
    public static long stringToLong(String s) {
	error = NO_ERROR;
	if (s==null) {
	    error = ARG_ERROR;
	    return (long)0;
	}
        try {
            return Long.parseLong(s);
        } catch(NumberFormatException nfe) {
            error = FORMAT_ERROR;
            return (long)0;
        }
    }
    /**
     * Reads a lexeme from the input and converts it into an <CODE>int</CODE>.
     * <p>
     * In case of an error, the error state is updated.
     * @return the next lexeme as an integer (<CODE>int</CODE>)
     */
    public static int readInt() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToInt(theTokenizer.currentToken());
    }
    /**
     * Reads a lexeme from the input and converts it into a <CODE>long</CODE>.
     * <p>
     * In case of an error, the error state is updated.
     * @return the next lexeme as a long integer (<CODE>long</CODE>)
     */
    public static long readLong() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToLong(theTokenizer.currentToken());
    }
    /**
     * Reads a lexeme from the input and converts it into a <CODE>float</CODE>.
     * <p>
     * In case of an error, the error state is updated.
     * @return the next lexeme as a single-precision float (<CODE>float</CODE>)
     */
    public static float readFloat() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToFloat(theTokenizer.currentToken());
    }
    /**
     * Reads a lexeme from the input and converts it into a <CODE>double</CODE>.
     * <p>
     * In case of an error, the error state is updated.
     * @return the next lexeme as a double-precision float (<CODE>double</CODE>)
     */
    public static double readDouble() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return 0;
	}
	return stringToDouble(theTokenizer.currentToken());
    }
    /**
     * Reads a lexeme from the input.
     * <p>
     * In case of an error, the error state is updated.
     * @return the next lexeme (<code>null</code> in case of failure).
     */
    public static String readString() {
        int tokenType;
	tokenType = theTokenizer.nextToken();
	if (tokenType==MyTokenizer.EOF) {
	    error = STREAM_ERROR;
	    return null;
	}
	error = NO_ERROR;
	return theTokenizer.currentToken();
    }
    /**
     * Prints a character.
     * @param c the character to print.
     */
    public static void print(char c) {
	System.out.print(c);
    }
    /**
     * Prints an integer (<code>int</code>)
     * @param i the integer to print
     */
    public static void print(int i) {
	System.out.print(i);
    }
    /**
     * Prints a boolean (<code>boolean</code>)
     * @param b the boolean to print
     */
    public static void print(boolean b) {
	System.out.print(b);
    }
    /**
     * Prints a long integer (<code>long</code>)
     * @param l the long integer to print
     */
    public static void print(long l) {
	System.out.print(l);
    }
    /**
     * Prints a single-precision floating point number (<code>float</code>)
     * @param f the float to print
     */
    public static void print(float f) {
	System.out.print(f);
    }
    /**
     * Prints a double-precision flooating point number (<code>double</code>)
     * @param d the double to print
     */
    public static void print(double d) {
	System.out.print(d);
    }
    /**
     * Prints the <code>String</code> representation of an object
     * @param o the object to print
     */
    public static void print(Object o) {
	System.out.print(o);
    }
    /**
     * Prints a character string (<code>String</code>)
     * @param s the string to print
     */
    public static void print(String s) {
	System.out.print(s);
    }
    /**
     * Prints a single character (<code>char</code>) followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param c the character to print
     */
    public static void println(char c) {
	System.out.println(c);
    }
    /**
     * Prints a boolean (<code>boolean</code>) followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param b the boolean to print
     */
    public static void println(boolean b) {
	System.out.println(b);
    }
    /**
     * Prints an integer (<code>int</code>) followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param i the integer to print
     */
    public static void println(int i) {
	System.out.println(i);
    }
    /**
     * Prints a long integer (<code>long</code>) followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param l the long integer to print
     */
    public static void println(long l) {
	System.out.println(l);
    }
    /**
     * Prints a single-precision floating point number (<code>float</code>)
     * followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param f the single-precision float to print
     */
    public static void println(float f) {
	System.out.println(f);
    }
    /**
     * Prints a double-precision floating point number (<code>double</code>)
     * followed by a newline.
     * A call to this method is equivalent to
     * <code>print(c); println();</code>.
     * @param f the double-precision float to print
     */
    public static void println(double d) {
	System.out.println(d);
    }
    /**
     * Prints the <code>String</code> representation of an object
     * followed by a newline.
     * @param o the object to print
     */
    public static void println(Object o) {
	System.out.println(o);
    }
    /**
     * Prints a character string (<code>String</code>)
     * followed by a newline.
     * @param s the string to print
     */
    public static void println(String s) {
	System.out.println(s);
    }
    /**
     * Prints a new line.
     */
    public static void println() {
	System.out.println();
    }
    /**
     * Extracts the character at a given position in a character string.
     * @param aString the character string 
     * @param position the index of the character to extract
     * @return the character at the given position
     */
    public static char charAt(String aString,int position) {
	return aString.charAt(position);
    }
    /**
     * Extracts a substring from a given character string.
     * @param aString the character string 
     * @param begin index of the first character of the substring
     * @param end index of the first character excluded from the substring
     * @return the extracted character string
     */
    public static String subString(String aString,int begin,int end) {
	return aString.substring(begin,end);
    }
    /**
     * Converts a boolean to a character string (<code>String</code>).
     * @param value the boolean (<code>boolean</code>)
     * @return the corresponding character string
    */
    public static String booleanToString(boolean value) {
	return String.valueOf(value);
    }
    /**
     * Converts a character to a character string (<code>String</code>).
     * @param value the character (<code>char</code>)
     * @return the corresponding character string
    */
    public static String charToString(char value) {
	return String.valueOf(value);
    }
    /**
     * Converts a single-precision floating point number to a character string (<code>String</code>).
     * @param value the single-precision floating number (<code>float</code>)
     * @return the corresponding character string
    */
    public static String floatToString(float value) {
	return String.valueOf(value);
    }
    /**
     * Converts a double-precision floating point number to a character string (<code>String</code>).
     * @param value the double-precision floating number (<code>double</code>)
     * @return the corresponding character string
    */
    public static String doubleToString(double value) {
	return String.valueOf(value);
    }
    /**
     * Converts an integer to a character string (<code>String</code>).
     * @param value the integer (<code>int</code>)
     * @return the corresponding character string
    */
    public static String intToString(int value) {
	return String.valueOf(value);
    }
    /**
     * Converts a long integer to a character string (<code>String</code>).
     * @param value the integer (<code>long</code>)
     * @return the corresponding character string
    */
    public static String longToString(long value) {
	return String.valueOf(value);
    }
    /**
     * Tests the equality between two character strings.
     * @param aString the first character string
     * @param anotherString the second character string
     * @return <code>true</code> if the two are equal,
     * <code>false</code> otherwise.
     */
    public static boolean equals(String aString,String anotherString) {
	return aString.equals(anotherString);
    }
    /**
     * Compares two character strings.
     * @param aString the first character string
     * @param anotherString the second character string
     * @return 0 if the strings are equal; a negative integer if the
     * second is bigger than the first using the lexicographic order
     * (dictionary order); a positive integer if the first string is
     * bigger than the second one.
     */
    public static int compare(String aString,String anotherString) {
	return aString.compareTo(anotherString);
    }
    /**
     * Computes the length of a given character string.
     * @param aString the character string
     * @return its length as an integer
     */
    public static int length(String aString) {
	return aString.length();
    }
    private static JFrame theFrame = null;
    private static Drawable theDrawable = null;
    /**
     * Prepares a drawing zone of size 200x200.
     */
    public static void startDrawings() {
	startDrawings(200,200);
    }
    /**
     * Prepares a drawing zone of arbitrary size.
     * @param width the width of the display
     * @param height the height of the display
     */
    public static void startDrawings(int width,int height) {
	if (theFrame!=null) return;
	theFrame = new JFrame("Drawing");
	theFrame.setResizable(false);
	theDrawable = new Drawable(width,height);
	theFrame.getContentPane().add(theDrawable);
	theFrame.pack();
	theFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	theFrame.setVisible(true);
	try {
	    Thread.sleep(2000); // Should be synchronized!!!
	} catch(Exception e) {
	}
    }
    /**
     * Closes the drawing window.
    */
    public static void stopDrawings() {
	if (theFrame==null) return;
	theFrame.dispose();
	theFrame=null;
    }
    /**
     * Draws a point in the drawing zone.
     * @param x the x-coordinate (abscissa)
     * @param y the y-coordinate (ordinate)
     */
    public static void drawPoint(int x,int y) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.drawLine(x,y,x,y);
    }
    /**
     * Sets the color of the subsequent drawing operations.
     * Colors are given using 256 <em>levels</em> of red, green and blue,
     * from <code>0,0,0</code> for black to <code>255,255,255</code> for white.
     * Red is therefore with <code>255,0,0</code>, green with <code>0,255,0</code>
     * and blue with <code>0,0,255</code>.
     * @param r the red level
     * @param g the green level
     * @param b the blue level
     */
    public static void setColor(int r,int g,int b) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.setColor(r,g,b);
    }
    /**
     * Sets the color of the subsequent drawing operations to be grey.
     * There are 256 grey <em>levels</em>,
     * from <code>0</code> for black to <code>255</code> for white.
     * An average grey would be done with parameter <code>127</code>.
     * @param c the grey-level
     */
    public static void setGray(int c) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.setColor(c,c,c);
    }
    /**
     * Draws a line between two points in the drawing area.
     * Dessine un segment de droite dans la zone d'affichage.
     * @param x0 the x-coordinate of the first point.
     * @param y0 the y-coordinate of the first point.
     * @param x1 the x-coordinate of the second point.
     * @param y1 the y-coordinate of the second point.
     */
    public static void drawLine(int x0,int y0,int x1, int y1) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.drawLine(x0,y0,x1,y1);
    }


    /**
     * Draws an empty rectangle in the display zone.
     * <p>
     * The top-left corner is at coordinates <code>x</code>
     * and <code>y</code>. 
     * @param x the x-coordinate of the top-left corner.
     * @param y the y-coordinate of the top-left corner.
     * @param w the width of the rectangle.
     * @param h the height of the rectangle.
     */
    public static void drawRect(int x,int y,int w, int h) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.drawRect(x,y,w,h);
    }

    /**
     * Draws a filled rectangle in the display zone.
     * <p>
     * The top-left corner is at coordinates <code>x</code>
     * and <code>y</code>. 
     * @param x the x-coordinate of the top-left corner.
     * @param y the y-coordinate of the top-left corner.
     * @param w the width of the rectangle.
     * @param h the height of the rectangle.
     */
    public static void fillRect(int x,int y,int w, int h) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.fillRect(x,y,w,h);
    }

    /**
     * Draws a circle in the display zone.
     * @param x the x-coordinate of the centre of the circle.
     * @param y the y-coordinate of the centre of the circle.
     * @param r the circle radius.
     */
    public static void drawCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.drawOval(x-r, y-r, 2*r, 2*r);
    }

    /**
     * Draws a filled circle in the display zone.
     * @param x the x-coordinate of the centre of the circle.
     * @param y the y-coordinate of the centre of the circle.
     * @param r the circle radius.
     */
    public static void fillCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.fillOval(x-r, y-r, 2*r, 2*r);
    }


    /**
     * Draws an arc in the display zone.
     * <p>
     * The angles are in degrees.
     * The direction is counter-clockwise.
     * The initial angle is <code>angleInit</code>
     * and the terminating angle is <code>angleInit + angleCouvert</code>.
     * @param x the x-coordinate of the centre of the circle.
     * @param y the y-coordinate of the centre of the circle.
     * @param r the circle radius.
     * @param angleInit the initial angle.
     * @param angleCouvert the angle covered by the arc.
     */
    public static void drawArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.drawArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Draws an filled arc in the display zone.
     * <p>
     * The angles are in degrees.
     * The direction is counter-clockwise.
     * The initial angle is <code>angleInit</code>
     * and the terminating angle is <code>angleInit + angleCouvert</code>.
     * @param x the x-coordinate of the centre of the circle.
     * @param y the y-coordinate of the centre of the circle.
     * @param r the circle radius.
     * @param angleInit the initial angle.
     * @param angleCouvert the angle covered by the arc.
     */
    public static void fillArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.fillArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Clears the diplay area completely.
    */
    public static void clearArea() {
	if (theFrame==null)
	    throw new RuntimeException("You need to call startDrawings() before drawing");
	theDrawable.clearArea();
    }
}

class Drawable extends Canvas {
    private int width, height;
    private Image backingStore;
    private Graphics backingGraphics;
    public Drawable(int width,int height) {
	setSize(width,height);
    }
    public void setSize(int width,int height) {
	super.setSize(width,height);
	this.width = width;
	this.height = height;
    }
    public Dimension getPreferredSize() {
	return new Dimension(width,height);
    }
    public void paint(Graphics g) {
	if (backingStore==null) {
	    backingStore = createImage(Drawable.this.width,
				       Drawable.this.height);
	    backingGraphics = backingStore.getGraphics();
	}
	g.drawImage(backingStore,0,0,this);
    }
    public void update(Graphics g) {
	paint(g);
    }
    public void drawLine(int x0,int y0,int x1, int y1) {
	if (backingGraphics==null) return;
	backingGraphics.drawLine(x0,y0,x1,y1);
	repaint();
    }
    
    public void drawRect(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.drawRect(x,y,l,h);
	repaint();
    }

    public void fillRect(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.fillRect(x,y,l,h);
	repaint();
    }

    public void drawOval(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.drawOval(x,y,l,h);
	repaint();
    }

    public void fillOval(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.fillOval(x,y,l,h);
	repaint();
    }

    public void drawArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	if(backingGraphics==null) return;
	backingGraphics.drawArc(x,y,l,h, angleInit, angleCouvert);
	repaint();
    }

    public void fillArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	if(backingGraphics==null) return;
	backingGraphics.fillArc(x,y,l,h, angleInit, angleCouvert);
	repaint();
    }

    public void setColor(int c1,int c2,int c3) {
	if (backingGraphics==null) return;
	backingGraphics.setColor(new Color(c1,c2,c3));
    }
    public void clearArea() {
	if (backingGraphics==null) return;
	backingGraphics.clearRect(0,0,width,height);
	repaint();
    }
    public void sleep(int secondes) {
        try {
            Thread.sleep(secondes*1000);
        } catch(InterruptedException ie) {}
    }
}

